@@ -1,10 +1,10 @@
+# frozen_string_literal: true
 require 'parallel_tests/rspec/logger_base'
 require 'parallel_tests/rspec/runner'
 
 class ParallelTests::RSpec::FailuresLogger < ParallelTests::RSpec::LoggerBase
   if RSPEC_2
-    def dump_failures(*args)
-    end
+    def dump_failures(*args); end
   else
     RSpec::Core::Formatters.register self, :dump_summary
   end
