@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'spec_helper'
 
 describe ParallelTests::Test::RuntimeLogger do
@@ -40,10 +41,12 @@ describe ParallelTests::Test::RuntimeLogger do
 
       # log looking good ?
       lines = File.read("tmp/parallel_runtime_test.log").split("\n").sort.map { |x| x.sub(/\d$/, "") }
-      expect(lines).to eq([
-        "test/0_test.rb:0.7",
-        "test/1_test.rb:0.7",
-      ])
+      expect(lines).to eq(
+        [
+          "test/0_test.rb:0.7",
+          "test/1_test.rb:0.7"
+        ]
+      )
     end
   end
 end
